package gui;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.geom.AffineTransform;
import java.util.LinkedList;
import java.util.List;

import util.DrawUtils;

// Klasa koja predstavja jednu labelu
public class GuiTextLabel implements Drawable {
	public static final String DEFAULTFONT = "Times New Roman";
	public static final int DEFAULTSTILE = Font.PLAIN;
	public static final Color DEFAULTCOLOR = Color.BLACK;

	protected int x;
	protected int y;
	protected String label;

	int fontSize;
	String fontName;
	int fontStyle;
	Color fontColor;
	double angle;

	public GuiTextLabel(int x, int y, String label, int alignment,
			int fontSize, String fontName, int fontStyle, Color fontColor,
			double angle) {
		this.x = DrawUtils.calcX(x, label, alignment, fontSize, fontName,
				fontStyle);
		this.y = y;
		this.label = label;
		this.fontSize = fontSize;
		this.angle = angle;
		this.fontName = fontName;
		this.fontStyle = fontStyle;
		this.fontColor = fontColor;

		rotate();
	}

	public GuiTextLabel(int x, int y, String label, int fontSize,
			int alignment, double angle) {
		this(x, y, label, alignment, fontSize, DEFAULTFONT, DEFAULTSTILE,
				DEFAULTCOLOR, angle);
	}

	public GuiTextLabel(int x, int y, String label, int fontSize, int alignment) {
		this(x, y, label, fontSize, alignment, 0);
	}

	public GuiTextLabel(int x, int y, String label, int fontSize) {
		this(x, y, label, fontSize, DrawUtils.ALIGNRIGHT, 0);
	}

	public GuiTextLabel(int x, int y, String label, int fontSize, double angle) {
		this(x, y, label, fontSize, DrawUtils.ALIGNRIGHT, angle);
	}

	private void rotate() {
		double xP = x * Math.cos(angle) + y * Math.sin(angle);
		double yP = -x * Math.sin(angle) + y * Math.cos(angle);
		this.x = (int) xP;
		this.y = (int) yP;
	}

	public void draw(Graphics g) {
		update();

		Color c = g.getColor();
		Font f = g.getFont();

		g.setColor(fontColor);
		g.setFont(new Font(fontName, fontStyle, fontSize));

		if (angle == 0) {
			g.drawString(label, x, y);
		} else {
			Graphics2D g2 = (Graphics2D) g;
			AffineTransform orig = g2.getTransform();
			g2.rotate(angle);
			g.drawString(label, x, y);
			g2.setTransform(orig);
		}

		g.setFont(f);
		g.setColor(c);
	}

	public void update() {
	}

	@Override
	public int getWidth() {
		return DrawUtils.stringWidth(label, fontSize, fontName, fontStyle);
	}

	@Override
	public int getHeight() {
		return DrawUtils.stringHeight(label, fontSize, fontName, fontStyle);
	}

	@Override
	public Point getCorner(int corner) {
		switch (corner) {
		case TOPLEFT:
			return new Point(x, y - getHeight());
		case TOPRIGHT:
			return new Point(x + getWidth(), y - getHeight());
		case BOTOMLEFT:
			return new Point(x, y);
		case BOTOMRIGHT:
			return new Point(x + getWidth(), y);
		}
		return null;
	}

	@Override
	public List<Drawable> getDrawables() {
		List<Drawable> result = new LinkedList<Drawable>();
		return result;
	}

}
