package gui.components;

import java.awt.Point;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import gui.Drawable;
import gui.GuiLine;
import gui.GuiPinLine;
import gui.GuiTextLabel;
import logic.components.DC;
import util.DrawUtils;

public class DCGui extends DrawableComponentAbstract {

	private static final int DCWIDTH = 180;

	Point[] inPortPositions, outPortPositions;

	public DCGui(DC dc, int x, int y, String[] args, boolean printOutPortName) {
		this(dc, x, y, args, true, printOutPortName);
	}

	public DCGui(DC dc, int x, int y, String[] args, boolean printInPortName, boolean printOutPortName) {
		super(dc, x, y, args, printInPortName, printOutPortName);

		inPortPositions = new Point[dc.getInPins().length];
		outPortPositions = new Point[dc.getOutPins().length];
	}

	private static final int DISTANCE = 15;

	@Override
	public List<Drawable> getDrawables(){
		List<Drawable> gui = new LinkedList<Drawable>();
		
		DC dc = (DC) component;

		GuiLine line; // Pomocna promenljiva
		List<List<Point>> sections;
		List<Point> points;

		int inNum = dc.getInPins().length;
		int outNum = (int) (Math.pow(2, inNum));
		int heihgt = (outNum + 1) * DISTANCE;

		GuiTextLabel label = new GuiTextLabel(x + 54, y + getYMargin(inNum) + heihgt / 2, args[1], 14);
		drawables.add(label);

		sections = new ArrayList<List<Point>>();
		points = new ArrayList<Point>();
		points.add(new Point(x + 39, y + getYMargin(inNum)));
		points.add(new Point(x + 95, y + getYMargin(inNum)));
		points.add(new Point(x + 95, y + getYMargin(inNum) + heihgt));
		points.add(new Point(x + 39, y + getYMargin(inNum) + heihgt));
		points.add(new Point(x + 39, y + getYMargin(inNum)));
		sections.add(points);
		drawables.add(new GuiLine(sections));

		int startI0 = getYMargin(inNum) + heihgt / 2 + inNum * DISTANCE / 2;

		for (int i = 0; i < inNum; i++) {
			sections = new ArrayList<List<Point>>();
			points = new ArrayList<Point>();
			points.add(new Point(x - 50 + 82, y + startI0));
			points.add(new Point(x - 50 + 89, y + startI0));
			sections.add(points);
			line = new GuiPinLine(sections, dc.getInPin(i));
			drawables.add(line);
			inPortPositions[i] = new Point(x - 50 + 89, y + startI0);

			label = new GuiTextLabel(x - 50 + 94, y + startI0, "" + i, 10);
			drawables.add(label);
			label = new GuiTextLabel(x + 28, y + startI0, args[4 + i], 13, DrawUtils.ALIGNLEFT);
			drawables.add(label);

			startI0 -= DISTANCE;
		}

		sections = new ArrayList<List<Point>>();
		points = new ArrayList<Point>();
		points.add(new Point(x - 50 + 117, y + getYEnd(inNum)));
		points.add(new Point(x - 50 + 117, y + getYEnd(inNum) + 8));
		sections.add(points);
		line = new GuiPinLine(sections, dc.getE());
		drawables.add(line);

		label = new GuiTextLabel(x - 50 + 116, y + getYEnd(inNum) - 6, "E", 10);
		drawables.add(label);
		label = new GuiTextLabel(x + 68, y + getYEnd(inNum) + 22, args[3], 13, DrawUtils.ALIGCENTER);
		drawables.add(label);

		startI0 = getYEnd(inNum) - DISTANCE;

		for (int i = 0; i < outNum; i++) {
			sections = new ArrayList<List<Point>>();
			points = new ArrayList<Point>();
			points.add(new Point(x - 50 + 145, y + startI0));
			points.add(new Point(x - 50 + 151, y + startI0));
			sections.add(points);
			line = new GuiPinLine(sections, dc.getOutPin(i));
			drawables.add(line);
			outPortPositions[i] = new Point(x - 50 + 151, y + startI0);

			label = new GuiTextLabel(x - 50 + 135, y + startI0 + 2, "" + i, 10);
			drawables.add(label);
			if (printOutPortName) {
				label = new GuiTextLabel(x - 50 + 155, y + startI0 + 2, args[4 + inNum + i], 13);
				drawables.add(label);
			}
			startI0 -= DISTANCE;
		}

		gui.addAll(drawables);
		return gui;
	}

	@Override
	public int getWidth() {
		return DCWIDTH;
	}

	@Override
	public int getHeight() {
		int max = component.getOutPins().length;
		if (max <= 8) {
			return 180;
		} else {
			return 180 * max / 8;
		}
	}

	protected int getYMargin(int n) {
		if (n == 2) {
			return 46;
		} else {
			return 18;
		}
	}

	protected int getYEnd(int n) {
		int result = getYMargin(n);
		result += ((int) (Math.pow(2, n)) + 1) * DISTANCE;
		return result;
	}

	@Override
	public Point getOutPortPosition(int index) {
		if (index >= 0 && index < outPortPositions.length) {
			return outPortPositions[index];
		}
		return null;
	}

	@Override
	public Point getInPortPosition(int index) {
		if (index >= 0 && index < inPortPositions.length) {
			return inPortPositions[index];
		}
		return null;
	}


}
