package gui.components;

import java.awt.Point;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import gui.Drawable;
import gui.GuiImageLabel;
import gui.GuiLine;
import gui.GuiPinLabel;
import gui.GuiPinLine;
import gui.GuiTextLabel;
import logic.components.TSB;

public class TSBGui extends OneOutGui {

	public TSBGui(TSB tsb, int x, int y, String[] args,
			boolean printInPortName, boolean printOutPortName) {
		super(tsb, x, y, args, printInPortName, printOutPortName,
				"src/images/TRI.png");

	}

	@Override
	public List<Drawable> getDrawables(){
		List<Drawable> gui = new LinkedList<Drawable>();

		GuiLine line;
		GuiTextLabel label;

		GuiImageLabel im = new GuiImageLabel(x + 100, y, imageURL);
		drawables.add(im);

		List<List<Point>> sections;
		List<Point> points;

		if (printInPortName) {
			label = getLabel(component.getInPin(0), x, y + 9);
			drawables.add(label);
		}
		if (printOutPortName) {
			String outName = component.getName();
			if ((outName != null) && !("".equals(outName))) {
				label = new GuiTextLabel(x + 140, y + 4, outName, 12);
				drawables.add(label);
			}
		}

		sections = new ArrayList<List<Point>>();
		points = new ArrayList<Point>();
		points.add(new Point(x + 66, y + 9));
		points.add(new Point(x + 100, y + 9));
		sections.add(points);
		line = new GuiPinLine(sections, component.getInPin(0));
		drawables.add(line);

		sections = new ArrayList<List<Point>>();
		points = new ArrayList<Point>();
		points.add(new Point(x + 125, y + 9));
		points.add(new Point(x + 160, y + 9));
		sections.add(points);
		line = new GuiPinLine(sections, component.getOutPin(0));
		drawables.add(line);

		TSB tsb = (TSB) component;
		sections = new ArrayList<List<Point>>();
		points = new ArrayList<Point>();
		points.add(new Point(x + 112, y + 15));
		points.add(new Point(x + 112, y + 27));
		sections.add(points);
		line = new GuiPinLine(sections, tsb.getE());
		drawables.add(line);

		drawables.add(new GuiPinLabel(x + 20, y + 12, component.getInPin(0)));

		drawables.add(new GuiTextLabel(x + 75, y + 8, ""
				+ component.getInPin(0).getNumOfLines(), 14));
		drawables.add(new GuiTextLabel(x + 145, y + 8, ""
				+ component.getOutPin(0).getNumOfLines(), 14));

		String eName = tsb.getE().getName();
		if ((eName != null) && !("".equals(eName))) {
			label = getLabel(tsb.getE(), x + 140, y + 40);
			drawables.add(label);
		}
		
		gui.addAll(drawables);
		return gui;
	}

	@Override
	public int getWidth() {
		String outName = component.getName();
		TSB tsb = (TSB) component;
		String eName = tsb.getE().getName();

		return Math.max(112 - 35 + eName.length() * 10,
				112 - 35 + outName.length() * 10);
	}

	@Override
	public int getHeight() {
		return 60;
	}

}
