package gui.components;

import java.awt.Point;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import gui.Drawable;
import gui.GuiLine;
import gui.GuiPinLine;
import gui.GuiTextLabel;
import logic.LogicalComponent;
import logic.Pin;
import logic.components.TwoOne;

public class TwoOneGui extends DrawableComponentAbstract {

	private static final int WIDTH = 200;

	OrGui orGui1, orGui2;
	AndGui andGui;

	LogicalComponent left, right;
	LogicalComponent center;

	public TwoOneGui(LogicalComponent component, int x, int y, String[] args, boolean printInPortName,
			boolean printOutPortName) {
		super(component, x, y, args, printInPortName, printOutPortName);

		TwoOne tcmp = (TwoOne) component;
		this.left = tcmp.getLeft();
		this.right = tcmp.getRight();
		this.center = tcmp.getCenter();

		orGui1 = new OrGui(left, x, y, args, printInPortName, false);
		orGui2 = new OrGui(right, x, y + orGui1.getHeight() + 20, args, printInPortName, false);
		int centarY = (orGui1.getHeight() + orGui2.getHeight()) / 2;
		andGui = new AndGui(center, orGui1.getOutPortPosition(0).x - 65, y + centarY - 25, args, false,
				printOutPortName);
	}

	@Override
	public List<Drawable> getDrawables(){
		List<Drawable> gui = new LinkedList<Drawable>();
		
		gui.addAll(orGui1.getDrawables());
		gui.addAll(orGui2.getDrawables());
		gui.addAll(andGui.getDrawables());
		
		GuiLine line; // Pomocna promenljiva

		List<List<Point>> sections;
		List<Point> points;

		Point p1 = orGui1.getOutPortPosition(0);
		Point p2 = orGui2.getOutPortPosition(0);
		Point p3 = andGui.getInPortPosition(0);
		Point p4 = andGui.getInPortPosition(1);

		sections = new ArrayList<List<Point>>();
		points = new ArrayList<Point>();
		points.add(new Point(p1.x, p1.y));
		points.add(new Point((p1.x + p3.x) / 2, p1.y));
		points.add(new Point((p1.x + p3.x) / 2, p3.y));
		points.add(new Point(p3.x, p3.y));
		sections.add(points);
		line = new GuiPinLine(sections, left.getOutPin(0));
		drawables.add(line);

		sections = new ArrayList<List<Point>>();
		points = new ArrayList<Point>();
		points.add(new Point(p2.x, p2.y));
		points.add(new Point((p2.x + p4.x) / 2, p2.y));
		points.add(new Point((p2.x + p4.x) / 2, p4.y));
		points.add(new Point(p4.x, p4.y));
		sections.add(points);
		line = new GuiPinLine(sections, right.getOutPin(0));
		drawables.add(line);

		gui.addAll(drawables);
		return gui;
	}

	@Override
	public int getWidth() {
		return WIDTH;
	}

	@Override
	public int getHeight() {
		int result = orGui1.getHeight() + 20 + orGui2.getHeight();
		return result;
	}

	protected static GuiTextLabel getLabel(Pin in, int x, int y) {
		GuiTextLabel label;
		String name = in.getName();
		if (name == null) {
			name = "";
		}
		label = new GuiTextLabel(moveString(name, 12, x), y, name, 12);
		return label;
	}

	private static int moveString(String text, int fontSize, int x) {
		int numChars = (2 * (text.length() + 1)) / 3;
		int result = x - numChars * (fontSize);

		return result;
	}

	@Override
	public Point getOutPortPosition(int index) {
		return andGui.getOutPortPosition(index);
	}

	@Override
	public Point getInPortPosition(int index) {
		// TODO Auto-generated method stub
		return null;
	}

}
