package logic;

import java.util.ArrayList;
import java.util.List;

public class Pin {

	private String name;

	private boolean isBool;

	private int numOfLines;
	private int mask;

	private boolean boolVal;
	private int intVal;

	private boolean highZ;

	private boolean driver;

	private List<LogicalComponent> children;

	public Pin(String name) {
		this(name, true, 1, 0, false, false);
	}

	public Pin(boolean val, String name) {
		this(name, true, 1, 0, val, false);
	}

	public Pin(int val, int brlin, String name) {
		this(name, false, brlin, val, false, false);
	}

	public Pin(String name, boolean isBool, int numOfLines, int intVal, boolean boolVal, boolean highZ) {
		this.name = name;
		this.isBool = isBool;
		this.numOfLines = numOfLines;
		this.mask = (int) ((1l << numOfLines) - 1);
		this.intVal = intVal;
		this.boolVal = boolVal;
		this.highZ = highZ;
		this.driver = !highZ;

		children = new ArrayList<LogicalComponent>();
	}

	public boolean isBool() {
		return isBool;
	}

	public void setIsInt() {
		isBool = false;
	}

	public void setIsBool() {
		isBool = true;
	}

	public void clear() {
		if (boolVal == true || intVal != 0) {
			boolVal = false;
			intVal = 0;
			signalAll();
		}
	}

	public boolean getBoolVal() {
		return boolVal;
	}

	public void setBoolVal(boolean bVal) {
		if (isBool) {
			if ((boolVal != bVal) || (highZ == true)) {
				this.boolVal = bVal;
				this.highZ = false;
				if (this.driver) {
					signalAll();
				}
			}
		}
	}

	public int getIntVal() {
		return intVal;
	}

	public void setIntVal(int iVal) {
		iVal = mask & iVal;
		if (!isBool) {
			if ((intVal != iVal) || (highZ == true)) {
				this.intVal = iVal;
				this.highZ = false;
				if (this.driver) {
					signalAll();
				}
			}
		}
	}

	public boolean isHighZ() {
		return highZ;
	}

	public boolean isDriver() {
		return driver;
	}

	public void setHighZ(boolean highZboolValue) {
		if ((highZ != highZboolValue)) {
			this.highZ = highZboolValue;
			this.driver = !highZboolValue;
			signalAll();
		}
	}

	public void setVal(Pin pin) {
		if ((boolVal != pin.boolVal) || (intVal != pin.intVal) || (highZ == true)) {
			this.boolVal = pin.boolVal;
			this.intVal = pin.intVal;
			this.highZ = pin.highZ;
			signalAll();
		}
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public int getNumOfLines() {
		return numOfLines;
	}

	public void setNumOfLines(int numOfLines) {
		if (numOfLines >= 0 && numOfLines <= 32) {
			this.numOfLines = numOfLines;
			this.mask = (int) ((1l << numOfLines) - 1);
		} else {
			throw new RuntimeException("Not suppotres port size greater then 32:" + numOfLines);
		}
	}

	public void addChild(LogicalComponent logcomp) {
		if (!children.contains(logcomp)) {
			children.add(logcomp);
		}
	}

	public List<LogicalComponent> getChildren() {
		return children;
	}

	void signalAll() {
		LogicalComponent child = null;
		for (int i = 0; i < children.size(); i++) {
			child = children.get(i);
			if (!child.isSeq())
				child.execute();
		}
	}

	public boolean compare(Pin pin) {
		boolean result = (pin != null) && (this.intVal == pin.intVal) && (this.boolVal == pin.boolVal)
				&& (this.highZ == pin.highZ);
		return result;
	}

	public void copyVal(Pin pin) {
		if (pin == null) {
			return;
		}
		this.intVal = pin.intVal;
		this.boolVal = pin.boolVal;
		this.highZ = pin.highZ;
	}

	public String toString() {
		return name + ":" + this.boolVal + ";" + this.intVal + ";" + this.highZ;
	}
}
