package logic.components;

import logic.LogicalComponent;
import logic.Pin;
import util.Log;
import util.Mask;

public class ALU extends LogicalComponent {

	private Pin add;
	private Pin sub;
	private Pin inc;
	private Pin dec;
	private Pin and;
	private Pin or;
	private Pin xor;
	private Pin not;

	private Pin CN;

	// private Pin C0;
	// private Pin asr;
	// private Pin transferA;

	private int size = 0;
	private int mask = 0xff;

	public ALU(int size) {
		super(2, 1);
		out[0].setIsInt();
		out[0].setNumOfLines(size);
		CN = new Pin(false, "CN");
		allPins.add(CN);
		checkSize(size);
	}

	public void checkSize(int size) {
		if (this.size > 32) {
			throw new RuntimeException("Maxima ALU size 32: " + this.size);
		}
		this.size = size;
		this.mask = Mask.createMask(size);

	}

	public void func() {
		int akc = 0;
		if ((add != null) && (add.getBoolVal())) {
			akc++;
		}
		if ((sub != null) && (sub.getBoolVal())) {
			akc++;
		}
		if ((inc != null) && (inc.getBoolVal())) {
			akc++;
		}
		if ((dec != null) && (dec.getBoolVal())) {
			akc++;
		}
		if ((and != null) && (and.getBoolVal())) {
			akc++;
		}
		if ((or != null) && (or.getBoolVal())) {
			akc++;
		}
		if ((xor != null) && (xor.getBoolVal())) {
			akc++;
		}
		if ((not != null) && (not.getBoolVal())) {
			akc++;
		}
		// if ((asr!=null) && (asr.getBoolVal())){
		// akc++;}
		// if ((transferA!=null) && (transferA.getBoolVal())){
		// akc++;}

		if (akc > 1) {
			Log.log(":Error!!! Multiple ALU operations active");
			out[0].setIntVal(0);
			CN.setBoolVal(false);
			return;
		}

		if (akc == 0) {
			out[0].setIntVal(0);
			CN.setBoolVal(false);
			return;
		}

		int A = in[0].getIntVal();
		if (in[0].isHighZ()) {
			A = 0;
		}
		int B = in[1].getIntVal();
		if (in[1].isHighZ()) {
			B = 0;
		}
		if ((add != null) && (add.getBoolVal())) {
			int result = A + B;
			result = result & mask;
			out[0].setIntVal(result);

			if (CN != null) {
				long al = (A) & 0xFFFFFFFFl;
				long bl = (B) & 0xFFFFFFFFl;
				long maskl = mask & 0xFFFFFFFFl;
				if ((al + bl) > maskl) {
					CN.setBoolVal(true);
				} else {
					CN.setBoolVal(false);
				}
			}
			return;
		}

		if ((sub != null) && (sub.getBoolVal())) {
			int result = A - B;
			result = result & mask;
			out[0].setIntVal(result);

			if (CN != null) {
				long al = (A) & 0xFFFFFFFFl;
				long bl = (((B ^ 0xFFFFFFFF) + 1l) & mask) & 0xFFFFFFFFl;
				long maskl = mask & 0xFFFFFFFFl;
				if ((al + bl) > maskl) {
					CN.setBoolVal(true);
				} else {
					CN.setBoolVal(false);
				}
			}
			return;
		}

		if ((inc != null) && (inc.getBoolVal())) {
			int result = A + 1;
			result = result & mask;
			out[0].setIntVal(result);

			if (CN != null) {
				long al = (A) & 0xFFFFFFFFl;
				long bl = 1l;
				long maskl = mask & 0xFFFFFFFFl;
				if ((al + bl) > maskl) {
					CN.setBoolVal(true);
				} else {
					CN.setBoolVal(false);
				}
			}
			return;
		}

		if ((dec != null) && (dec.getBoolVal())) {
			int result = A - 1;
			result = result & mask;
			out[0].setIntVal(result);

			if (CN != null) {
				long al = (A) & 0xFFFFFFFFl;
				long bl = (mask) & 0xFFFFFFFFl;
				long maskl = mask & 0xFFFFFFFFl;
				if ((al + bl) > maskl) {
					CN.setBoolVal(true);
				} else {
					CN.setBoolVal(false);
				}
			}
			return;
		}

		if ((and != null) && (and.getBoolVal())) {
			int result = A & B;
			result = result & mask;
			out[0].setIntVal(result);
			CN.setBoolVal(false);
			return;
		}

		if ((or != null) && (or.getBoolVal())) {
			int result = A | B;
			result = result & mask;
			out[0].setIntVal(result);
			CN.setBoolVal(false);
			return;
		}

		if ((xor != null) && (xor.getBoolVal())) {
			int result = A ^ B;
			result = result & mask;
			out[0].setIntVal(result);
			CN.setBoolVal(false);
			return;
		}

		if ((not != null) && (not.getBoolVal())) {
			int result = A ^ 0xFFFFFFFF;
			result = result & mask;
			out[0].setIntVal(result);
			CN.setBoolVal(false);
			return;
		}

		// if ((asr!=null) && (asr.getBoolVal())) {
		// int pom = A;
		// A >>= 1;
		// out[0].setIntVal(A);
		// if(C0!=null)
		// C0.setBoolVal(pom % 2 == 1);
		// return;
		// }
		// if ((transferA!=null) && (transferA.getBoolVal())) {
		// out[0].setIntVal(in[0].getIntVal());CN.setBoolVal(false);
		// return;
		// }

	}

	public void setPinX(Pin pin) {
		setInPin(0, pin);
	}

	public void setPinY(Pin pin) {
		setInPin(1, pin);
	}

	public void setPinAdd(Pin add) {
		this.add = add;
		add.addChild(this);
	}

	public void setPinSub(Pin sub) {
		this.sub = sub;
		sub.addChild(this);
	}

	public void setPinInc(Pin inc) {
		this.inc = inc;
		inc.addChild(this);
	}

	public void setPinDec(Pin dec) {
		this.dec = dec;
		dec.addChild(this);
	}

	public void setPinAnd(Pin and) {
		this.and = and;
		and.addChild(this);
	}

	public void setPinOr(Pin or) {
		this.or = or;
		or.addChild(this);
	}

	public void setPinXor(Pin xor) {
		this.xor = xor;
		xor.addChild(this);
	}

	public void setPinNot(Pin not) {
		this.not = not;
		not.addChild(this);
	}

	// public void setPinC0(Pin c0) {
	// this.C0 = c0;
	// c0.addChild(this);
	// }
	// public void setPinAsr(Pin asr) {
	// this.asr = asr;
	// asr.addChild(this);
	// }
	// public void setPinTransferX(Pin transferX) {
	// this.transferX = transferX;
	// transferX.addChild(this);
	// }

	public Pin getPinC_Last() {
		return CN;
	}

	@Override
	public void initArgs(String[] args) {
		this.args = args;
		this.name = args[1];
	}

}
